/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2013 UJF-Grenoble 1, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/

#include "NiftiImageComponent.h"

//-- Qt
#include <QString>
#include <QTextStream>
#include <QFileInfo>
#include <QMessageBox>

//-- Vtk
#include <vtkImageData.h>

//-- Itk
#include <itkImageFileReader.h>
#include <itkOrientImageFilter.h>
#include <itkImageToVTKImageFilter.h>
#include <itkNiftiImageIO.h>
#include <itkImage.h>

// -------------------- constructor --------------------
NiftiImageComponent::NiftiImageComponent(const QString& fileName) throw(AbortException)
        : ImageComponent(fileName) {
    createComponent(fileName);
}

// -------------------- createComponent --------------------
void NiftiImageComponent::createComponent(const QString & filename) {
    if (!filename.isEmpty()) {
        vtkSmartPointer<vtkImageData> img = readVolume(filename);
        if (img) {
            setImageData(img, false);
            // use file basename as default name
            setName(QFileInfo(filename).baseName());
			//setName(filename);
        }
        else {
            throw (AbortException("Could not open file\n"));
        }
    }
    else {
        throw (AbortException("No filename found\n"));
    }

}

// -------------------- readVolume --------------------
vtkSmartPointer<vtkImageData> NiftiImageComponent::readVolume(const QString& filename) {
    
    typedef short                           PixelType;
    const   unsigned int                    Dimension = 3;
    typedef itk::Image< PixelType, Dimension >  ImageType;
    typedef itk::ImageFileReader< ImageType >  ReaderType;
    typedef itk::ImageToVTKImageFilter< ImageType > ConnectorType;
	
    // To copy data
    int dims[3];
    int extent[6];
    double origin[3];
    double spacing[3];

	itk::NiftiImageIO::Pointer l_NiftiIO = itk::NiftiImageIO::New();
	bool IsNativeImageNifti = l_NiftiIO->itk::NiftiImageIO::CanReadFile(filename.toStdString().c_str());
	if (IsNativeImageNifti){

		vtkSmartPointer<vtkImageData> oldPointer = NULL;
		vtkSmartPointer<vtkImageData> newImage = NULL;

		ReaderType::Pointer reader = ReaderType::New();
		// To convert itkImages to vtkImages
		ConnectorType::Pointer connector = ConnectorType::New();

		reader->SetFileName( filename.toStdString().c_str() );
		reader->SetImageIO(l_NiftiIO);
		reader->Update();

		connector->SetInput(reader->GetOutput());

		try {
			connector->Update();

			// Copy the output image because the filters will be removed
			// and we want to keep the image for other vtk pipelines
			oldPointer = connector->GetOutput();
			newImage = vtkSmartPointer<vtkImageData>::New();

			oldPointer->GetDimensions(dims);
			oldPointer->GetExtent(extent);
			oldPointer->GetOrigin(origin);
			oldPointer->GetSpacing(spacing);

			newImage->SetDimensions(dims);
			newImage->SetExtent(extent);
			newImage->SetOrigin(origin);
			newImage->SetSpacing(spacing);

			newImage->DeepCopy(oldPointer);

			// set the original slices. Other orientation are build when/if required only.
			newImage->Update();
		} catch ( itk::ExceptionObject & err ) {
			std::cerr << "ExceptionObject caught !" << std::endl;
			std::cerr << err << std::endl;
			return NULL;
		}

		oldPointer = NULL;

		return newImage;

	}
	return 0;
}


